package jp.co.atmarkit.quiz.web.logic
{
import jp.co.atmarkit.quiz.cnst.AppConsts;
import jp.co.atmarkit.quiz.cnst.MessageConsts;
import jp.co.atmarkit.quiz.components.AnswerPage;
import jp.co.atmarkit.quiz.components.QuizPanel;
import jp.co.atmarkit.quiz.entity.Comment;
import jp.co.atmarkit.quiz.entity.Quiz;
import jp.co.atmarkit.quiz.logic.Logic;
import jp.co.atmarkit.quiz.web.QuizForm;

import flash.events.MouseEvent;
import flash.net.URLRequestMethod;
import flash.net.URLVariables;

import mx.controls.Alert;
import mx.effects.Move;
import mx.events.EffectEvent;
import mx.events.FlexEvent;
import mx.managers.PopUpManager;
import mx.rpc.AsyncToken;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
    
    
/**
 * クイズアプリケーションのロジッククラス
 */
public class QuizLogic extends Logic {
//--------------------------------------
//  Variables
//--------------------------------------
    /** サーバータイプ */
    private var serverType:uint = 0; // 777：スタンドアロン、0：Struts1
    
    /** Quiz のインデックス */
    private var questionNumber:uint = 0;

    /** Quiz Panel インスタンス格納配列 */
    private var panelArray:Array = new Array();
    
    /** Move インスタンス（旧） */
    private var moveBefore:Move = new Move();

    /** Move インスタンス（新） */
    private var moveAfter:Move = new Move();

    /** テンポラリ QuizPanel インスタンス */
    private var tmpQuizPanel:QuizPanel;

    /** エフェクトロック */
    private var isEffect:Boolean;
    
    /** 設問のリスト */
    public var quizList:Array;
    
    /** コメントのリスト */
    public var commentList:Array;
    
//--------------------------------------
//  Constructor
//--------------------------------------
    /**
     * コンストラクタ
     */
    public function QuizLogic() {
    }

//--------------------------------------
//  Initialization
//--------------------------------------
    /**
     * onCreationCompleteHandler
     * 
     * @param event FlexEvent
     */
    override protected function onCreationCompleteHandler(event:FlexEvent):void {
        // クイズの設問をサーバーから取得する
        getQuizList();
    }
    
    /**
     * クイズパネルの初期化
     */
    private function initPanel():void {
        var index:uint;
        var indexLimit:uint = quizList.length;
        var newQuizPanel:QuizPanel;
        var myX:uint;
        var panelVisible:Boolean;
        
        // パネルの初期化
        for(index = 0; index < indexLimit; index++) {
            newQuizPanel = new QuizPanel();
            view.quizBase.addChild(newQuizPanel);
            
            // 最初のパネルだけ座標とvisibleを他のものと別に設定する
            if(index == 0) {
                myX = 0;
                panelVisible = true;
            } else {
                myX = 480;
                panelVisible = false;
            }
            
            // 各パネルの初期設定
            setPanel(newQuizPanel, myX, String(AppConsts.QUIZ_TITLE_PREFIX).concat(index + 1),
                quizList[index].question, 
                quizList[index].selector,
                quizList[index].correct,
                panelVisible);
        }
        
        // 戻るボタンのクリックイベントハンドラ
        view.previousButton.addEventListener(MouseEvent.CLICK, previousHandler, false, 0, true);

        // 進むボタンのクリックイベントハンドラ
        view.nextButton.addEventListener(MouseEvent.CLICK, nextHandler, false, 0, true);

        // 採点ボタンのクリックイベントハンドラ
        view.markButton.addEventListener(MouseEvent.CLICK, markHandler, false, 0, true);
    }

    /**
     * Panel の初期設定
     * 
     * @param aObj 設定対象 QuizPanel インスタンス
     * @param aX QuizPanel の X 座標
     * @param aTitle クイズのタイトル
     * @param aBody クイズの本文
     * @param aSelector 選択肢
     * @param aVisible QuizPanel.visible 値（初期表示設問以外はすべて false）
     */
    private function setPanel(aObj:QuizPanel, aX:int, aTitle:String, aBody:String, aSelector:Array, aCorrect:Object, aVisible:Boolean = false):void {
        aObj.x = aX;
        aObj.questionTitleText = aTitle;
        aObj.questionBodyText = aBody;
        aObj.questionSelector = aSelector;
        aObj.visible = aVisible;
		aObj.correct = aCorrect;
        panelArray.push(aObj);
    }

	/**
	 * パネルに解答をセット 
	 * 
	 * @param questionNumber 解答番号
	 */	
	private function setAnswer(questionNumber:uint):void {
		var nowPanel:QuizPanel = panelArray[questionNumber] as QuizPanel;

		if (nowPanel.vs.selectedIndex == 0) {
			nowPanel.answer = nowPanel.answerText.text;
		} else {
			nowPanel.answer = nowPanel.selectorGroup.selectedValue;
		}
	}

//--------------------------------------
//  UI Event Handler
//--------------------------------------
    /**
     * previous ボタン押下処理
     * 
     * @param event MouseEvent
     */
    private function previousHandler(event:MouseEvent):void {
        // エフェクト中はボタン押下処理は無効
        if(isEffect) {
            return;
        }
        
        moveEffect(panelArray[questionNumber], panelArray[questionNumber - 1], 640);
        questionNumber--;
        view.nextButton.enabled = (quizList.length != questionNumber) ? true : false;
        view.previousButton.enabled = (0 == questionNumber) ? false : true;
        view.markButton.visible = (quizList.length - 1 == questionNumber) ? true : false;
    }

    /**
     * next ボタン押下処理
     * 
     * @param event MouseEvent
     */
    private function nextHandler(event:MouseEvent):void{
        // エフェクト中はボタン押下処理は無効
        if(isEffect) {
            return;
        }

		setAnswer(questionNumber);

        moveEffect(panelArray[questionNumber], panelArray[questionNumber + 1], -640);
        questionNumber++;
        view.nextButton.enabled = (quizList.length - 1 == questionNumber) ? false : true;
        view.previousButton.enabled = (0 <= questionNumber) ? true : false;
        view.markButton.visible = (quizList.length - 1 == questionNumber) ? true : false;
    }

    /**
     * 採点ボタン押下処理
     * 
     * @param event MouseEvent
     */
	private function markHandler(event:MouseEvent):void {
        // エフェクト中はボタン押下処理は無効
        if(isEffect) {
            return;
        }

		setAnswer(questionNumber);

        var answerPage:AnswerPage = new AnswerPage();
        answerPage.panels = panelArray;
        
        // コメント情報を結果画面に渡す
        answerPage.commentList = commentList;
        
        PopUpManager.addPopUp(answerPage, this.view, true);
	}


//--------------------------------------
//  Effect
//--------------------------------------

    /**
     * 画面遷移エフェクト
     * 
     * @param oldTarget エフェクト前に表示されていた QuizPanel インスタンス
     * @param newTarget エフェクト後に表示される QuizPanel インスタンス
     * @param aXTo 移動座標
     */
    private function moveEffect(oldTarget:QuizPanel, newTarget:QuizPanel, aXTo:Number):void {
        var _fromX:Number = oldTarget.x;
        var _toX:Number = _fromX + aXTo
        var _duration:Number = 500;
        isEffect = true;
        newTarget.visible = true;
        tmpQuizPanel = oldTarget;
        
        with(moveBefore) {
            duration       = _duration;
            xTo            = _toX;
            target         = oldTarget;
            
            // エフェクト完了イベントハンドラ
            addEventListener(EffectEvent.EFFECT_END, effectEndHandler, false, 0, true);
        }
        
        with(moveAfter) {
            duration       = _duration;
            xTo            = _fromX;
            target         = newTarget;
        }
        
        moveBefore.play();
        moveAfter.play();
    }
    
    /**
     * エフェクト完了イベントハンドラ
     * 
     * @param event EffectEvent
     */
    private function effectEndHandler(event:EffectEvent):void {
        tmpQuizPanel.visible = false;
        isEffect = false;
    }

//--------------------------------------
//  Server Connection Event Handler
//--------------------------------------
    
    /** 
     * クイズの設問をサーバーから取得する
     * このメソッドの挙動がサーバーの種類に応じて異なります
     */
    private function getQuizList():void {
        // サーバー側がSAStrutsの場合
        if(serverType == AppConsts.SERVER_TYPE_STRUTS1) {
            getQuizListStruts1();
        } else if(serverType == AppConsts.SERVER_TYPE_LOCAL) {
            getQuizListLocal();
        }
    }

    /** 
     * クイズの設問をサーバーから取得する：Struts版
     */
    private function getQuizListStruts1():void {
        // HTTPService基本設定
        view.srvStruts1Quiz.url = AppConsts.SERVER_URL_STRUTS1_QUIZ;
        view.srvStruts1Quiz.method = URLRequestMethod.POST;
        view.srvStruts1Quiz.addEventListener(ResultEvent.RESULT, onGetQuizListStruts1_result);
        view.srvStruts1Quiz.addEventListener(FaultEvent.FAULT, onGetQuizListStruts1_fault);

        // HTTPServiceリクエストパラメータ設定
        // 特にパラメータは設定していません
        var variables:URLVariables = new URLVariables();

        view.srvStruts1Quiz.request = variables;
        
        // リクエストの実行（トークンを保持しておく）
        var at:AsyncToken = view.srvStruts1Quiz.send();
        
        // トークンは通信周りのイベントハンドラ共用する場合、
        // どの処理からの結果/失敗イベントかを判断するのに利用できます
        at.tokenId = AppConsts.SERVER_TYPE_STRUTS1;
    }
    
    /** 
     * コメントをサーバーから取得する：Struts版
     */
    private function getCommentListStruts1():void {
        // HTTPService基本設定
        view.srvStruts1Comment.url = AppConsts.SERVER_URL_STRUTS1_COMMENT;
        view.srvStruts1Comment.method = URLRequestMethod.POST;
        view.srvStruts1Comment.addEventListener(ResultEvent.RESULT, onGetCommentListStruts1_result);
        view.srvStruts1Comment.addEventListener(FaultEvent.FAULT, onGetCommentListStruts1_fault);

        // HTTPServiceリクエストパラメータ設定
        // 特にパラメータは設定していません
        var variables:URLVariables = new URLVariables();

        view.srvStruts1Comment.request = variables;
        
        // リクエストの実行（トークンを保持しておく）
        var at:AsyncToken = view.srvStruts1Comment.send();
        
        // トークンは通信周りのイベントハンドラ共用する場合、
        // どの処理からの結果/失敗イベントかを判断するのに利用できます
        at.tokenId = AppConsts.SERVER_TYPE_STRUTS1;
    }

    /** 
     * クイズの設問をサーバーから取得する：Struts1版：成功イベント
     */
    private function onGetQuizListStruts1_result(event:ResultEvent):void {
        // レスポンスのXMLをQuizオブジェクトのリストに変換
        createQuizListFromXML(XML(event.result));
        
        // 引き続きコメントをサーバーから取得する
        getCommentListStruts1();
    }

    /** 
     * クイズの設問をサーバーから取得する：Struts1版：失敗イベント
     */
    private function onGetQuizListStruts1_fault(event:FaultEvent):void {
        trace(event.fault.message);
        Alert.show(MessageConsts.ERROR_SERVER_ERROR);
    }

    /** 
     * コメントをサーバーから取得する：Struts1版：成功イベント
     */
    private function onGetCommentListStruts1_result(event:ResultEvent):void {
        // レスポンスのXMLをオブジェクトのリストに変換
        createCommnetListFromXML(XML(event.result));

        // パネルの初期化
        initPanel();
        
        // 次へボタンを有効化
        view.nextButton.enabled = true;
    }

    /** 
     * コメントをサーバーから取得する：Struts1版：失敗イベント
     */
    private function onGetCommentListStruts1_fault(event:FaultEvent):void {
        trace(event.fault.message);
        Alert.show(MessageConsts.ERROR_SERVER_ERROR);
    }

    /** 
     * クイズの設問をサーバーから取得する：ローカル版
     */
    private function getQuizListLocal():void {
        createQuizList();
        
        // 引き続きコメントを取得
        createCommentList();
        
        // パネルの初期化
        initPanel();

        // 次へボタンを有効化
        view.nextButton.enabled = true;
    }

//--------------------------------------
//  Functions
//--------------------------------------
    
    /** 
     * XMLクイズデータのオブジェクト化
     * 
     * @param xml XML
     */
    private function createQuizListFromXML(xml:XML):void {
        quizList = new Array();
        var quizXML:XML;
        var quiz:Quiz;
        
        for each (quizXML in xml.quiz) {
            trace(quizXML);
            quiz = Quiz.populate(quizXML)
            quizList.push(quiz);
        }
    }
    
    /** 
     * XMLコメントデータのオブジェクト化 
     * 
     * @param xml XML
     */
    private function createCommnetListFromXML(xml:XML):void {
        commentList = new Array();
        var commentXML:XML;
        var comment:Comment;
        
        for each (commentXML in xml.comment) {
            trace(commentXML);
            comment = Comment.populate(commentXML)
            commentList.push(comment);
        }
    }

    /** 
     * ローカルでのクイズデータの作成 
     */
    private function createQuizList():void {
        quizList = new Array();
        
        var quiz:Quiz = new Quiz();
        quiz.question = "ActionScriptにおいて、\n\tvar a:uint = 0;\n\ttrace(a++);\nの出力結果を答えよ";
        quiz.selectors = null;
        quiz.correct = "0";
        quizList.push(quiz);
        
        quiz = new Quiz();
        quiz.question = "ActionScriptにおいて、\n\tvar o:Object = new Object();\n\to.a = 1;\n\to.f = function():void\n\t{\n\t\ttrace(this.a);\n\t}\n\tvar f:Function = o.f;\n\tf();\nの出力結果を答えよ";
        quiz.selectors = null;
        quiz.correct = "undefined";
        quizList.push(quiz);
        
        quiz = new Quiz();
        quiz.question = "DTOとは何の略か";
        quiz.selectors = null;
        quiz.correct = "Data Transfer Object";
        quizList.push(quiz);
        
        quiz = new Quiz();
        quiz.question = "以下の演算子のうち、ActionScriptで利用できないものはどれか？";
        quiz.selectors = ">>>=,===,<<<=,::";
        quiz.correct = "<<<=";
        quizList.push(quiz);
        
        quiz = new Quiz();
        quiz.question = "Flexのイベントでのイベント段階に関して正しい順番は？";
        quiz.selectors = "キャプチャ->ターゲット->バブリング,ターゲット->キャプチャ->バブリング,バブリング->ターゲット->キャプチャ,バブリング->キャプチャ->ターゲット";
        quiz.correct = "キャプチャ->ターゲット->バブリング";
        quizList.push(quiz);
        
        quiz = new Quiz();
        quiz.question = "Flexで使える3大通信方式、HTTPService、WebService、後一つは？";
        quiz.selectors = null;
        quiz.correct = "RemoteObject";
        quizList.push(quiz);

        quiz = new Quiz();
        quiz.question = "株価状況を可視化するアプリケーションを作りたい。CandlestickChart以外に使うと有効なチャートはどれか？";
        quiz.selectors = "ColumnChart,PieChart,HLOCチャート";
        quiz.correct = "HLOCチャート";
        quizList.push(quiz);

        quiz = new Quiz();
        quiz.question = "プリミティブ型以外のプロパティを持つエンティティのコレクションを特になんという";
        quiz.selectors = null;
        quiz.correct = "階層コレクション";
        quizList.push(quiz);

        quiz = new Quiz();
        quiz.question = "Stringクラスで正規表現が使えるメソッドは4つある。match()、replace()、search()と後一つは何か？";
        quiz.selectors = null;
        quiz.correct = "split()";
        quizList.push(quiz);

        quiz = new Quiz();
        quiz.question = "IMEConversionModeで対応している言語は、日本語、中国語とあと1言語ある。何か？";
        quiz.selectors = null;
        quiz.correct = "韓国語";
        quizList.push(quiz);
    }

    /** 
     * ローカルでのコメントデータの作成 
     */
    private function createCommentList():void {
        commentList = new Array();
        
        var comment:Comment = new Comment();
        comment.comment = "がんばって得点をとりましょう。\nファイト♪";
        commentList.push(comment);
        
        comment = new Comment();
        comment.comment = "もとがんばって高得点をとりましょう。\nファイト♪";
        commentList.push(comment);
        
        comment = new Comment();
        comment.comment = "がんばりましたね♪えらいえらいp(^-^q)";
        commentList.push(comment);
    }
        
//--------------------------------------
//  View-Logic Binding
//--------------------------------------
    /** 画面 */
    public var _view:QuizForm;
    
    /**
     * 画面を取得します
     */     
    public function get view():QuizForm {
        if (_view == null) {
            _view = super.document as QuizForm;
        }
        return _view;
    }
    
    /**
     * 画面をセットします。
     * 
     * @param view セットする画面
     */
    public function set view(view:QuizForm):void {
        _view = view;
    }
}
}